#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <sys/time.h>
#include <unistd.h>
#include <string.h>
#include <Evas.h>
#include <Ecore.h>
#include <Imlib2.h>
#include <config.h>

#define MAX_EVAS_COLORS (216)
#define MAX_FONT_CACHE (512 * 1024)
#define MAX_IMAGE_CACHE (16 * (1024 * 1024))
#define FONT_DIRECTORY PACKAGE_DATA_DIR"/data/fonts/"
#define IMAGE_DIRECTORY PACKAGE_DATA_DIR"/data/images/"
#define RENDER_ENGINE RENDER_METHOD_ALPHA_SOFTWARE
#define FN FONT_DIRECTORY
#define IM IMAGE_DIRECTORY
#define W 400
#define H 300
/* #define RENDER_ENGINE RENDER_METHOD_BASIC_HARDWARE */
/* #define RENDER_ENGINE RENDER_METHOD_3D_HARDWARE */

typedef struct _image Image;

struct _image
{
   char *file;
   pid_t generator;
   char *thumb;
   Evas_Object o_thumb;
   int subst;
};

/* general functions */
double get_time (void);
void setup(void);

/* callbacks for evas handling */
/* when the event queue goes idle call this */
void e_idle(void *data);
/* when the window gets exposed call this */
void e_window_expose(Eevent * ev);
/* when the mouse moves in the window call this */
void e_mouse_move(Eevent * ev);
/* when a mouse button goes down in the window call this */
void e_mouse_down(Eevent * ev);
/* when a mouse button is released in the window call this */
void e_mouse_up(Eevent * ev);
/* when the window gets moved or resized */
void e_window_configure(Eevent * ev);
/* when a ket gets pressed */
void e_key_down(Eevent * ev);

void
show_panel (void *_data, Evas _e, Evas_Object _o, int _b, int _x, int _y);
void
hide_panel (void *_data, Evas _e, Evas_Object _o, int _b, int _x, int _y);
void
next_image(void *_data, Evas _e, Evas_Object _o, int _b, int _x, int _y);
void
next_image_up(void *_data, Evas _e, Evas_Object _o, int _b, int _x, int _y);
void
next_image_move(void *_data, Evas _e, Evas_Object _o, int _b, int _x, int _y);

void e_handle_resize(void);
void e_fix_icons(void);

void e_fade_info_in(int v, void *data);
void e_fade_info_out(int v, void *data);
void e_fade_scroller_in(int v, void *data);
void e_fade_scroller_out(int v, void *data);

/* globals */
Evas_Object o_bg;
Evas_Object o_logo;
Evas_Object o_panel;
Evas_Object o_showpanel;
Evas_Object o_hidepanel;
Evas_Object o_showbuttons;
Evas_Object o_arrow_l;
Evas_Object o_arrow_r;
Evas_Object o_arrow_t;
Evas_Object o_arrow_b;
Evas_Object o_image = NULL;
Evas_Object o_mini_image = NULL;
Evas_Object o_mini_select;
Evas_Object o_list_select;
Evas_Object o_panel_arrow_u;
Evas_Object o_panel_arrow_d;
Evas_Object o_bt_close;
Evas_Object o_bt_expand;
Evas_Object o_bt_full;
Evas_Object o_bt_next;
Evas_Object o_bt_prev;
Evas_Object o_bt_zoom_in;
Evas_Object o_bt_zoom_normal;
Evas_Object o_bt_zoom_out;
Evas_Object o_txt_info[10];
Evas_Object o_txt_tip[5];

char txt_info[2][4096];

Evas_List images = NULL;
Evas_List current_image = NULL;
Image *generating_image = NULL;

Evas evas;
Evas_Render_Method render_method = RENDER_ENGINE;
int max_colors = MAX_EVAS_COLORS;
int smoothness = 1;
int win_w = W, win_h = H;
int win_x = 0, win_y = 0;
Window main_win;

int icon_x = 0;
int icon_y = 0;

int panel_active = 0;
int buttons_active = 0;
int scroll_x = 0, scroll_y = 0;
int scroll_sx = 0, scroll_sy = 0;
int arrow_l = 0, arrow_r = 0, arrow_t = 0, arrow_b = 0;
int down_x = 0, down_y = 0;
int down_sx = 0, down_sy = 0;
double scale = 1.0;
double focus_pos = 0.0;
int need_thumbs = 1;
   

/* timers to do anims */

void
e_scroll_list(int v, void *data)
{
   int dd, d, fp;
      
   fp = (int)focus_pos - (win_h / 2);
   dd = d = fp + icon_y;
   if (dd < 0) dd = -dd;
   icon_y -= (d / 4);
   e_fix_icons();
   if (dd > 3)
     e_add_event_timer("e_scroll_list()", 0.05, e_scroll_list, v + 1, NULL);
}

void
e_fade_info_in(int v, void *data)
{
   int i, j;
   static double start = 0.0;
   double duration = 0.5;
   double val;
   
   if (v == 0) start = get_time();
   val = (get_time() - start) / duration;
   if (val > 1.0) val = 1.0;
   
   for (j = 0; j < 2; j++)
     {
	for (i = 0; i < 5; i++)
	  {
	     evas_show(evas, o_txt_info[(j * 5) + i]);
	     evas_set_layer(evas, o_txt_info[(j * 5) + i], 2000);
	     evas_set_text(evas, o_txt_info[(j * 5) + i], txt_info[j]);
	     if      (i == 0)
	       evas_move(evas, o_txt_info[(j * 5) + i], 0, (j * 20) + 1);
	     else if (i == 1)
	       evas_move(evas, o_txt_info[(j * 5) + i], 1, (j * 20));
	     else if (i == 2)
	       evas_move(evas, o_txt_info[(j * 5) + i], 2, (j * 20) + 1);
	     else if (i == 3)
	       evas_move(evas, o_txt_info[(j * 5) + i], 1, (j * 20) + 2);
	     else if (i == 4)
	       evas_move(evas, o_txt_info[(j * 5) + i], 1, (j * 20) + 1);
	     if (i == 4)
	       evas_set_color(evas, o_txt_info[(j * 5) + i], 255, 255, 255, (val * 255));
	     else
	       evas_set_color(evas, o_txt_info[(j * 5) + i], 0, 0, 0, (val * 255));
	  }
     }
   
   if (val < 1.0)
     e_add_event_timer("e_fade_info_in()", 0.05, e_fade_info_in, v + 1, data);
   else e_fade_info_out(0, NULL);
}

void
e_fade_info_out(int v, void *data)
{
   int i, j;
   static double start = 0.0;
   double duration = 8.0;
   double val;
   
   if (v == 0) start = get_time();
   val = (get_time() - start) / duration;
   if (val > 1.0) val = 1.0;
   val = 1.0 - val;

   for (j = 0; j < 2; j++)
     {
	for (i = 0; i < 5; i++)
	  {
	     if (val > 0.0)
	       evas_show(evas, o_txt_info[(j * 5) + i]);
	     else
	       evas_hide(evas, o_txt_info[(j * 5) + i]);
	     evas_set_layer(evas, o_txt_info[(j * 5) + i], 2000);
	     evas_set_text(evas, o_txt_info[(j * 5) + i], txt_info[j]);
	     if      (i == 0)
	       evas_move(evas, o_txt_info[(j * 5) + i], 0, (j * 20) + 1);
	     else if (i == 1)
	       evas_move(evas, o_txt_info[(j * 5) + i], 1, (j * 20));
	     else if (i == 2)
	       evas_move(evas, o_txt_info[(j * 5) + i], 2, (j * 20) + 1);
	     else if (i == 3)
	       evas_move(evas, o_txt_info[(j * 5) + i], 1, (j * 20) + 2);
	     else if (i == 4)
	       evas_move(evas, o_txt_info[(j * 5) + i], 1, (j * 20) + 1);
	     if (i == 4)
	       evas_set_color(evas, o_txt_info[(j * 5) + i], 255, 255, 255, (val * 255));
	     else
	       evas_set_color(evas, o_txt_info[(j * 5) + i], 0, 0, 0, (val * 255));
	  }
     }
   
   if (val < 1.0)
     e_add_event_timer("e_fade_info_in()", 0.05, e_fade_info_out, v + 1, NULL);
}

void
e_fade_scroller_in(int v, void *data)
{
   int i;
   static double start = 0.0;
   double duration = 0.5;
   double val;
   
   if (v == 0) start = get_time();
   val = (get_time() - start) / duration;
   if (val > 1.0) val = 1.0;
   
   evas_set_color(evas, o_mini_image, 255, 255, 255, (val * 255));
   evas_set_color(evas, o_mini_select, 255, 255, 255, (val * 255));
   
   if (val < 1.0)
     e_add_event_timer("e_fade_logo_in()", 0.05, e_fade_scroller_in, v + 1, data);
   if (data)
     e_fade_scroller_out(0, NULL);
}

void
e_fade_scroller_out(int v, void *data)
{
   int i;
   static double start = 0.0;
   double duration = 2.0;
   double val;
   
   if (v == 0) start = get_time();
   val = (get_time() - start) / duration;
   if (val > 1.0) val = 1.0;
   val = 1.0 - val;
   evas_set_color(evas, o_mini_image, 255, 255, 255, (val * 255));
   evas_set_color(evas, o_mini_select, 255, 255, 255, (val * 255));
   
   if (val < 1.0)
     e_add_event_timer("e_fade_logo_in()", 0.05, e_fade_scroller_out, v + 1, NULL);
}

void
e_fade_logo_in(int v, void *data)
{
   int i;
   static double start = 0.0;
   double duration = 1.0;
   double val;
   
   if (v == 0) start = get_time();
   val = (get_time() - start) / duration;
   
   evas_set_color(evas, o_logo, 255, 255, 255, (val * 255));
   
   if (val < 1.0)
     e_add_event_timer("e_fade_logo_in()", 0.05, e_fade_logo_in, v + 1, NULL);
}

void
e_fade_l_arrow_in(int v, void *data)
{
   int i;
   static double start = 0.0;
   double duration = 0.5;
   double val;
   
   if (arrow_l == 2) return;
   if (v == 0) start = get_time();
   val = (get_time() - start) / duration;
   
   evas_set_color(evas, o_arrow_l, 255, 255, 255, (int)(val * 255));
   
   if (val < 1.0)
     {
	e_add_event_timer("e_fade_l_arrow_in()", 0.05, e_fade_l_arrow_in, v + 1, NULL);
	arrow_l = 1;
     }
   else
     arrow_l = 2;
   evas_show(evas, o_arrow_l);
}

void
e_fade_l_arrow_out(int v, void *data)
{
   int i;
   static double start = 0.0;
   double duration = 0.5;
   double val;
   
   if (arrow_l == 0) return;
   if (v == 0) start = get_time();
   val = (get_time() - start) / duration;
   
   evas_set_color(evas, o_arrow_l, 255, 255, 255, ((1.0 - val) * 255));
   
   if (val < 1.0)
     {
	e_add_event_timer("e_fade_l_arrow_out()", 0.05, e_fade_l_arrow_out, v + 1, NULL);
	arrow_l = 1;
     }
   else
     {
	evas_hide(evas, o_arrow_l);
	arrow_l = 0;
     }
}

void
e_fade_r_arrow_in(int v, void *data)
{
   int i;
   static double start = 0.0;
   double duration = 0.5;
   double val;
   
   if (arrow_r == 2) return;
   if (v == 0) start = get_time();
   val = (get_time() - start) / duration;
   
   evas_set_color(evas, o_arrow_r, 255, 255, 255, (int)(val * 255));
   
   if (val < 1.0)
     {
	e_add_event_timer("e_fade_r_arrow_in()", 0.05, e_fade_r_arrow_in, v + 1, NULL);
	arrow_r = 1;
     }
   else
     arrow_r = 2;
   evas_show(evas, o_arrow_r);
}

void
e_fade_r_arrow_out(int v, void *data)
{
   int i;
   static double start = 0.0;
   double duration = 0.5;
   double val;
   
   if (arrow_r == 0) return;
   if (v == 0) start = get_time();
   val = (get_time() - start) / duration;
   
   evas_set_color(evas, o_arrow_r, 255, 255, 255, ((1.0 - val) * 255));
   
   if (val < 1.0)
     {
	e_add_event_timer("e_fade_r_arrow_out()", 0.05, e_fade_r_arrow_out, v + 1, NULL);
	arrow_r = 1;
     }
   else
     {
	evas_hide(evas, o_arrow_r);
	arrow_r = 0;
     }
}

void
e_fade_t_arrow_in(int v, void *data)
{
   int i;
   static double start = 0.0;
   double duration = 0.5;
   double val;
   
   if (arrow_t == 2) return;
   if (v == 0) start = get_time();
   val = (get_time() - start) / duration;
   
   evas_set_color(evas, o_arrow_t, 255, 255, 255, (int)(val * 255));
   
   if (val < 1.0)
     {
	e_add_event_timer("e_fade_t_arrow_in()", 0.05, e_fade_t_arrow_in, v + 1, NULL);
	arrow_t = 1;
     }
   else
     arrow_t = 2;
   evas_show(evas, o_arrow_t);
}

void
e_fade_t_arrow_out(int v, void *data)
{
   int i;
   static double start = 0.0;
   double duration = 0.5;
   double val;
   
   if (arrow_t == 0) return;
   if (v == 0) start = get_time();
   val = (get_time() - start) / duration;
   
   evas_set_color(evas, o_arrow_t, 255, 255, 255, ((1.0 - val) * 255));
   
   if (val < 1.0)
     {
	e_add_event_timer("e_fade_t_arrow_out()", 0.05, e_fade_t_arrow_out, v + 1, NULL);
	arrow_t = 1;
     }
   else
     {
	evas_hide(evas, o_arrow_t);
	arrow_t = 0;
     }
}

void
e_fade_b_arrow_in(int v, void *data)
{
   int i;
   static double start = 0.0;
   double duration = 0.5;
   double val;
   
   if (arrow_b == 2) return;
   if (v == 0) start = get_time();
   val = (get_time() - start) / duration;
   
   evas_set_color(evas, o_arrow_b, 255, 255, 255, (int)(val * 255));
   
   if (val < 1.0)
     {
	e_add_event_timer("e_fade_b_arrow_in()", 0.05, e_fade_b_arrow_in, v + 1, NULL);
	arrow_b = 1;
     }
   else
     arrow_b = 2;
   evas_show(evas, o_arrow_b);
}

void
e_fade_b_arrow_out(int v, void *data)
{
   int i;
   static double start = 0.0;
   double duration = 0.5;
   double val;
   
   if (arrow_b == 0) return;
   if (v == 0) start = get_time();
   val = (get_time() - start) / duration;
   
   evas_set_color(evas, o_arrow_b, 255, 255, 255, ((1.0 - val) * 255));
   
   if (val < 1.0)
     {
	e_add_event_timer("e_fade_b_arrow_out()", 0.05, e_fade_b_arrow_out, v + 1, NULL);
	arrow_b = 1;
     }
   else
     {
	evas_hide(evas, o_arrow_b);
	arrow_b = 0;
     }
}

void
e_slide_panel_in(int v, void *data)
{
   int i;
   static double start = 0.0;
   double duration = 0.5;
   double val;
   double px;
   int w;
   
   panel_active = 1;
   
   if (v == 0)
     evas_set_layer(evas, o_showpanel, 180);   
   if (v == 0) start = get_time();
   val = (get_time() - start) / duration;
   
   evas_get_image_size(evas, o_panel, &w, NULL);
   px =  (w * sin(val * 0.5 * 3.141592654)) - w;
   evas_move(evas, o_panel, px, 0);
   evas_move(evas, o_panel_arrow_u, px, 0);
   evas_move(evas, o_panel_arrow_d, px, win_h - 32);

   icon_x = (int)px;
   e_fix_icons();
   
   if (val < 1.0)
     e_add_event_timer("e_slide_panel()", 0.05, e_slide_panel_in, v + 1, NULL);
}

void
e_slide_panel_out(int v, void *data)
{
   int i;
   static double start = 0.0;
   double duration = 0.5;
   double val;
   double px;
   int w;
   
   if (v == 0)
     evas_set_layer(evas, o_showpanel, 1000);   
   if (v == 0) start = get_time();
   val = (get_time() - start) / duration;
   
   evas_get_image_size(evas, o_panel, &w, NULL);
   px =  (w * sin((1.0 - val) * 0.5 * 3.141592654)) - w;
   evas_move(evas, o_panel, px, 0);
   evas_move(evas, o_panel_arrow_u, px, 0);
   evas_move(evas, o_panel_arrow_d, px, win_h - 32);
   
   icon_x = px;
   e_fix_icons();
   
   if (val < 1.0)
     e_add_event_timer("e_slide_panel()", 0.05, e_slide_panel_out, v + 1, NULL);
   else
     panel_active = 0;     
}

void
e_slide_buttons_in(int v, void *data)
{
   int i;
   static double start = 0.0;
   double duration = 0.5;
   double val;
   double px;
   
   buttons_active = 1;
   
   if (v == 0) start = get_time();
   val = (get_time() - start) / duration;
   
   px =  win_w - 256 - ((256 * sin(val * 0.5 * 3.141592654)) - 256);

   evas_move(evas, o_bt_prev, px + 0, 0);
   evas_move(evas, o_bt_next, px + 32, 0);
   evas_move(evas, o_bt_zoom_normal, px + 64, 0);
   evas_move(evas, o_bt_zoom_in, px + 96, 0);
   evas_move(evas, o_bt_zoom_out, px + 128, 0);
   evas_move(evas, o_bt_expand, px + 160, 0);
   evas_move(evas, o_bt_full, px + 192, 0);
   evas_move(evas, o_bt_close, px + 224, 0);
   
   if (val < 1.0)
     e_add_event_timer("e_slide_buttons()", 0.05, e_slide_buttons_in, v + 1, NULL);
}

void
e_slide_buttons_out(int v, void *data)
{
   int i;
   static double start = 0.0;
   double duration = 0.5;
   double val;
   double px;
   
   if (v == 0) start = get_time();
   val = (get_time() - start) / duration;
   
   px = win_w - 256 - ((256 * sin((1.0 - val) * 0.5 * 3.141592654)) - 256);

   evas_move(evas, o_bt_prev, px + 0, 0);
   evas_move(evas, o_bt_next, px + 32, 0);
   evas_move(evas, o_bt_zoom_normal, px + 64, 0);
   evas_move(evas, o_bt_zoom_in, px + 96, 0);
   evas_move(evas, o_bt_zoom_out, px + 128, 0);
   evas_move(evas, o_bt_expand, px + 160, 0);
   evas_move(evas, o_bt_full, px + 192, 0);
   evas_move(evas, o_bt_close, px + 224, 0);
   
   if (val < 1.0)
     e_add_event_timer("e_slide_buttons()", 0.05, e_slide_buttons_out, v + 1, NULL);
   else
     buttons_active = 0;
}

Image *
e_image_new(char *file)
{
   Image *im;
   
   im = malloc(sizeof(Image));
   im->file = strdup(file);
   im->generator = 0;
   im->thumb = NULL;
   im->o_thumb = NULL;
   im->subst = 0;
   return im;
}

void
e_image_free(Image *im)
{
   if (im->file) free(im->file);
   if (im->thumb) free(im->thumb);
   free(im);
}

time_t
e_file_modified_time(char *file)
{
   struct stat         st;
   
   if (stat(file, &st) < 0) return 0;
   return st.st_mtime;
}

char *
e_file_home(void)
{
   static char *home = NULL;
   
   if (home) return home;
   home = getenv("HOME");
   if (!home) home = getenv("TMPDIR");
   if (!home) home = "/tmp";
   return home;
}

int
e_file_mkdir(char *dir)
{
   if (mkdir(dir, S_IRUSR | S_IWUSR | S_IXUSR | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH) < 0) return 0;
   return 1;
}

int
e_file_exists(char *file)
{
   struct stat         st;
   
   if (stat(file, &st) < 0) return 0;
   return 1;
}

int
e_file_is_dir(char *file)
{
   struct stat         st;
   
   if (stat(file, &st) < 0) return 0;
   if (S_ISDIR(st.st_mode)) return 1;
   return 0;
}

void
e_mkdirs(char *path)
{
   char ss[1024];
   int  i, ii;
   
   i = 0;
   ii = 0;
   while (path[i])
     {
	ss[ii++] = path[i];
	ss[ii] = 0;
	if (path[i] == '/')
	  {
	     if (!e_file_exists(ss))
	       e_file_mkdir(ss);
	     else if (!e_file_is_dir(ss))
	       return;
	  }
	i++;
     }
}

void
e_child(Eevent * ev)
{
   Ev_Child      *e;
   
   e = (Ev_Child *)ev->event;
   if (generating_image)
     {
	if (generating_image->generator == e->pid)
	  {
	     generating_image->generator = 0;
	     generating_image = NULL;
	  }
     }
}

void
e_do_thumb(char *file, char *thumb)
{
   Imlib_Image im;
   
   im = imlib_load_image(file);
   if (im)
     {
	Imlib_Image im_thumb;
	int w, h, iw, ih;
	
	imlib_context_set_image(im);
	w = imlib_image_get_width();
	h = imlib_image_get_height();
	imlib_context_set_anti_alias(1);
	iw = 1;
	ih = 1;
	if (w > h)
	  {
	     if (w > 48)
	       {
		  iw = 48;
		  ih = (48 * h) / w;
	       }
	     else
	       {
		  iw = w;
		  ih = h;
	       }
	  }
	else
	  {
	     if (h > 48)
	       {
		  ih = 48;
		  iw = (48 * w) / h;
	       }
	     else
	       {
		  ih = h;
		  iw = w;
	       }
	  }
	im_thumb = imlib_create_cropped_scaled_image(0, 0, w, h, iw, ih);
	imlib_context_set_image(im_thumb);
	imlib_image_set_format("argb");
	e_mkdirs(thumb);
	imlib_save_image(thumb);
     }
}

void
e_generate_thumb(Image *im)
{
   static int initted = 0;
   char buf[4096];
   
   if (generating_image) return;
   if (im->generator > 0) return;
   if (!initted)
     {
	e_event_filter_handler_add(EV_CHILD, e_child);
	initted = 1;
     }
   sprintf(buf, "%s/.entice/thumbs/%s._.argb", e_file_home(), im->file);
   if (im->thumb) free(im->thumb);
   im->thumb = strdup(buf);
   generating_image = im;
   im->generator = fork();
   if (im->generator != 0)
     {
	if (im->generator > 0)
	  {
	  }
	else
	  {
/*	     printf("Eeeeek - help. Can't fork???\n");*/
	  }
	return;
     }  
   else
     {
	if ((im->thumb) && (im->file))
	  {
	     if (e_file_modified_time(im->thumb) > 
		 e_file_modified_time(im->file)) exit(0);
	     e_do_thumb(im->file, im->thumb);
	  }
	exit(0);
     }
}

void
e_handle_arrows(void)
{
   if (o_image)
     {
	double ix, iy, iw, ih;
	
	evas_get_geometry(evas, o_image, &ix, &iy, &iw, &ih);	
	if (ix < 0) e_fade_l_arrow_in(0, NULL);
	else e_fade_l_arrow_out(0, NULL);
	if (iy < 0) e_fade_t_arrow_in(0, NULL);
	else e_fade_t_arrow_out(0, NULL);
	if (iw + ix > win_w) e_fade_r_arrow_in(0, NULL);
	else e_fade_r_arrow_out(0, NULL);
	if (ih + iy > win_h) e_fade_b_arrow_in(0, NULL);
	else e_fade_b_arrow_out(0, NULL);
     }
   else
     {
	e_fade_l_arrow_out(0, NULL);
	e_fade_r_arrow_out(0, NULL);
	e_fade_t_arrow_out(0, NULL);
	e_fade_b_arrow_out(0, NULL);
     }
}

void
e_display_current_image(void)
{
   scroll_x = 0;
   scroll_y = 0;
   scroll_sx = 0;
   scroll_sy = 0;
   if (o_mini_image)
     {
	evas_del_object(evas, o_mini_image);
	o_mini_image = NULL;
     }
   if (current_image)
     {
	char title[4096];
	
	if (o_image)
	  {
	     evas_del_object(evas, o_image);
	     o_image = NULL;
	  }
	o_image = evas_add_image_from_file(evas, 
					   ((Image *)(current_image->data))->file);
	evas_callback_add(evas, o_image, CALLBACK_MOUSE_DOWN, next_image, NULL);
	evas_callback_add(evas, o_image, CALLBACK_MOUSE_UP, next_image_up, NULL);
	evas_callback_add(evas, o_image, CALLBACK_MOUSE_MOVE, next_image_move, NULL);
	evas_show(evas, o_image);
	if (evas_get_image_load_error(evas, o_image) != IMLIB_LOAD_ERROR_NONE)
	  {
	     sprintf(txt_info[0], "Error LoadingFile: %s", ((Image *)(current_image->data))->file);
	     sprintf(txt_info[1], "");
	     sprintf(title, "Entice (Error Loading): %s",
		     ((Image *)(current_image->data))->file);
	     e_window_set_title(main_win, title);
	     evas_del_object(evas, o_image);
	     o_image = NULL;
	  }
	else
	  {
	     int w, h;
	     
	     evas_get_image_size(evas, o_image, &w, &h);
	     sprintf(txt_info[0], "File: %s", ((Image *)(current_image->data))->file);
	     sprintf(txt_info[1], "Size: %ix%i", w, h);
	     e_fade_info_in(0, NULL);
	     
	     sprintf(title, "Entice: %s",
		     ((Image *)(current_image->data))->file);
	     e_window_set_title(main_win, title);	     
	  }
     }
   else
     {
	e_window_set_title(main_win, "Entice (No Image)");
	evas_del_object(evas, o_image);
	o_image = NULL;	
     }
   if ((o_image) && (current_image))
     {
	o_mini_image = evas_add_image_from_file(evas,
						((Image *)(current_image->data))->file);
     }
   e_handle_resize();
   e_fix_icons();
   e_scroll_list(0, NULL);
   e_fade_scroller_in(0, (void *)1);
}

void
e_handle_resize(void)
{
   int w, h;
   
   evas_move(evas, o_bg, 0, 0);
   evas_resize(evas, o_bg, win_w, win_h);
   evas_set_layer(evas, o_bg, 0);
   
   evas_get_image_size(evas, o_logo, &w, &h);
   evas_move(evas, o_logo, (win_w - w) / 2, (win_h - h) / 2);
   evas_resize(evas, o_logo, w, h);
   evas_set_layer(evas, o_logo, 100);

   evas_get_image_size(evas, o_panel, &w, NULL);
   if (!panel_active)
     evas_move(evas, o_panel, -w, 0);
   evas_resize(evas, o_panel, w, win_h);
   evas_set_layer(evas, o_panel, 200);
   evas_set_layer(evas, o_panel_arrow_u, 250);
   evas_set_layer(evas, o_panel_arrow_d, 250);
   
   evas_move(evas, o_showpanel, 0, 0);
   evas_resize(evas, o_showpanel, 64, win_h);
   if (panel_active)
     evas_set_layer(evas, o_showpanel, 180);
   else
     evas_set_layer(evas, o_showpanel, 1000);
     {   
	double px = 0;
	
	if (buttons_active)
	  px = win_w - 256;	
	else
	  px = win_w;
	evas_move(evas, o_bt_prev, px + 0, 0);
	evas_move(evas, o_bt_next, px + 32, 0);
	evas_move(evas, o_bt_zoom_normal, px + 64, 0);
	evas_move(evas, o_bt_zoom_in, px + 96, 0);
	evas_move(evas, o_bt_zoom_out, px + 128, 0);
	evas_move(evas, o_bt_expand, px + 160, 0);
	evas_move(evas, o_bt_full, px + 192, 0);
	evas_move(evas, o_bt_close, px + 224, 0);
     }
   evas_move(evas, o_showbuttons, win_w - 256, 0);
   evas_resize(evas, o_showbuttons, 256, 32);
   evas_set_layer(evas, o_showbuttons, 1100);
   evas_set_layer(evas, o_bt_close, 1300);
   evas_set_layer(evas, o_bt_expand, 1300);
   evas_set_layer(evas, o_bt_full, 1300);
   evas_set_layer(evas, o_bt_next, 1300);
   evas_set_layer(evas, o_bt_prev, 1300);
   evas_set_layer(evas, o_bt_zoom_in, 1300);
   evas_set_layer(evas, o_bt_zoom_normal, 1300);
   evas_set_layer(evas, o_bt_zoom_out, 1300);
   
   evas_move(evas, o_hidepanel, 128, 0);
   evas_resize(evas, o_hidepanel, win_w - 128, win_h);
   evas_set_layer(evas, o_hidepanel, 1000);
   
   if (o_image)
     {
	int sx, sy, mx, my, pw, ph;
	
	sx = scroll_x;
	sy = scroll_y;
	evas_get_image_size(evas, o_image, &w, &h);
	pw = w;
	ph = h;
	w = (int)((double)w / scale);
	h = (int)((double)h / scale);
	if (w > win_w)
	  {
	     if (sx > ((w - win_w) / 2)) sx = ((w - win_w) / 2);
	     if (sx < -((w - win_w + 1) / 2)) sx = -((w - win_w + 1) / 2);
	  }
	else sx = 0;
	if (h > win_h)
	  {
	     if (sy > ((h - win_h) / 2)) sy = ((h - win_h) / 2);
	     if (sy < -((h - win_h + 1) / 2)) sy = -((h - win_h + 1) / 2);
	  }
	else sy = 0;
	if (o_mini_image)
	  {
	     int ww, hh, sw, sh;
	     
	     ww = 48;
	     hh = (48 * ph) / pw;
	     if (h > w)
	       {
		  hh = 48;
		  ww = (48 * pw) / ph;
	       }
	     evas_resize(evas, o_mini_image, ww, hh);	     
	     evas_set_image_fill(evas, o_mini_image, 0, 0, ww, hh);
	     evas_move(evas, o_mini_image, win_w - ww - 3, win_h - hh - 3);
	     evas_set_image_border(evas, o_mini_select, 4, 4, 4, 4);
	     sw = ww;
	     sh = hh;
	     if (win_w < w) sw = (win_w * ww) / w;
	     if (win_h < h) sh = (win_h * hh) / h;
	     mx = (-sx * ww) / pw;
	     my = (-sy * hh) / ph;
	     evas_resize(evas, o_mini_select, sw + 6, sh + 6);
	     evas_set_image_fill(evas, o_mini_select, 0, 0, sw + 6, sh + 6);
	     evas_move(evas, o_mini_select, 
		       win_w - ww - 3 - 3 + ((ww - sw) / 2) + mx,
		       win_h - hh - 3 - 3 + ((hh - sh) / 2) + my);
	     evas_set_layer(evas, o_mini_image, 110);
	     evas_set_layer(evas, o_mini_select, 111);
	     evas_show(evas, o_mini_image);
	     evas_show(evas, o_mini_select);
	  }
	evas_move(evas, o_image, 
		  sx + ((win_w - w) / 2), 
		  sy + ((win_h - h) / 2));
	evas_set_image_fill(evas, o_image, 0, 0, w, h);
	evas_resize(evas, o_image, w, h);
	evas_set_layer(evas, o_image, 100);	
	
	scroll_sx = sx;
	scroll_sy = sy;
	
	if ((win_w <= w) && 
	    (win_h <= h) && 
	    (!evas_get_image_alpha(evas, o_image)))
	  evas_hide(evas, o_bg);
	else
	  evas_show(evas, o_bg);	
	evas_hide(evas, o_logo);
     }
   else
     {
	evas_show(evas, o_bg);
	evas_show(evas, o_logo);
	e_fade_logo_in(0, NULL);
     }
   evas_get_image_size(evas, o_arrow_l, &w, &h);
   evas_move(evas, o_arrow_l, 0, (win_h - h) / 2);
   evas_get_image_size(evas, o_arrow_r, &w, &h);
   evas_move(evas, o_arrow_r, win_w - w, (win_h - h) / 2);
   evas_get_image_size(evas, o_arrow_t, &w, &h);
   evas_move(evas, o_arrow_t, (win_w - w) / 2, 0);
   evas_get_image_size(evas, o_arrow_b, &w, &h);
   evas_move(evas, o_arrow_b, (win_w - w) / 2, win_h - h);
   evas_set_layer(evas, o_arrow_l, 230);
   evas_set_layer(evas, o_arrow_r, 230);
   evas_set_layer(evas, o_arrow_t, 230);
   evas_set_layer(evas, o_arrow_b, 230);

   e_handle_arrows();
}

void
e_toggle_fullscreen(void)
{
   static int full = 0;
   static Window win = 0;
   
   if (!full)
     {
	Window ewin;
	int rw, rh;
	
	e_grab();
	e_sync();
	e_window_get_geometry(0, NULL, NULL, &rw, &rh);
	win = e_window_override_new(0, 0, 0, rw, rh);
	e_window_set_events(win, XEV_CONFIGURE);
	ewin = evas_get_window(evas);
	e_window_reparent(ewin, win, 0, 0);
	e_window_resize(ewin, rw, rh);
	evas_set_output_viewport(evas, 0, 0, rw, rh);
	evas_set_output_size(evas, rw, rh);
	win_w = rw;
	win_h = rh;
	e_handle_resize();
	if (panel_active)
	  e_slide_panel_out(0, NULL);
	if (buttons_active)
	  e_slide_buttons_out(0, NULL);
	e_window_show(win);
	e_keyboard_grab(ewin);
	e_pointer_grab(ewin, CurrentTime);
	full = 1;
     }   
   else
     {
	Window ewin;
	
	e_ungrab();
	e_sync();
	e_pointer_ungrab(CurrentTime);
	e_keyboard_ungrab();
	e_window_get_geometry(main_win, NULL, NULL, &win_w, &win_h);
	ewin = evas_get_window(evas);
	e_window_reparent(ewin, main_win, 0, 0);
	if (win) e_window_destroy(win);
	win = 0;
	e_window_resize(ewin, win_w, win_h);
	evas_set_output_viewport(evas, 0, 0, win_w, win_h);
	evas_set_output_size(evas, win_w, win_h);
	e_handle_resize();
	if (panel_active)
	  e_slide_panel_out(0, NULL);
	if (buttons_active)
	  e_slide_buttons_out(0, NULL);
	full = 0;
     }
}

void
e_fix_icons(void)
{
   Evas_List l;
   int i, im_no;
   double pos;
   
   i = 1;
   im_no = 0;
   for (l = images; l; l = l->next, i++)
     {
	if (l == current_image) im_no = i;
     }
   i = 1;
   pos = 0.0;
   for (l = images; l; l = l->next, i++)
     {
	Image *im;
	double w, h;
	double sc;
	double space;
	int d;
	
	im = l->data;
	w = 48; h = 48;
	if (im->thumb)
	  {
	     int iw, ih;
	     
	     evas_get_image_size(evas, im->o_thumb, &iw, &ih);
	     w = iw; h = ih;
	  }
	d = im_no - i;
	if (d < 0) d = -d;
	sc = 1.0 / (1.0 + (((double)d) * 0.2));
	if (sc < 0.333333) sc = 0.333333;
	w *= sc;
	h *= sc;
	space = 48 * sc;
	evas_resize(evas, im->o_thumb, w, h);
	evas_set_image_fill(evas, im->o_thumb, 0, 0, w, h);		    
	evas_move(evas, im->o_thumb, 
		  2 + ((48 - w) / 2) + icon_x, 
		  2 + pos + ((space - h) / 2) + icon_y);
	if (d == 0) focus_pos = pos + (space / 2) + 1;
	pos += space + 2;
     }
   evas_set_layer(evas, o_list_select, 240);
   evas_show(evas, o_list_select);
   evas_move(evas, o_list_select, icon_x, focus_pos - 2 + icon_y - (48 / 2));
   evas_resize(evas, o_list_select, 52, 52);
   evas_set_image_fill(evas, o_list_select, 0, 0, 52, 52);
   evas_set_image_border(evas, o_list_select, 4, 4, 4, 4);
}

/* callbacks */
void
e_list_click(void *_data, Evas _e, Evas_Object _o, int _b, int _x, int _y)
{
   current_image = _data;
   e_display_current_image();
}

void
e_list_item_zoom(int v, void *data)
{
   Evas_List l;
   Image *im;
   static Evas_Object zo = NULL;
   static double t_start = 0.0;
   double t;
   
   l = data;
   im = l->data;
   if (v == -1)
     {
	evas_del_object(evas, zo);
	zo = NULL;
	return;
     }
   if (v == 0)
     {
	t_start = e_get_time();
        if ((im->thumb) && (im->file))
	  {
	     zo = evas_add_image_from_file(evas, im->thumb);
	  }
     }
   t = (e_get_time() - t_start) * 2;
   if (t > 1.0) t = 1.0;
   if (zo)
     {
	double x, y, w, h, nw, nh;
	int iw, ih;
	
	evas_get_image_size(evas, im->o_thumb, &iw, &ih);
	evas_get_geometry(evas, im->o_thumb, &x, &y, &w, &h);
	evas_set_layer(evas, zo, 300);
	nw = (double)iw * t * 2;
	nh = (double)ih * t * 2;
	evas_move(evas, zo, x + w + (t * 32), y + (h - nh) / 2);
	evas_resize(evas, zo, nw, nh);
	evas_set_image_fill(evas, zo, 0, 0, nw, nh);
	evas_set_color(evas, zo, 255, 255, 255, (int)(t * 255));
	evas_show(evas, zo);
     }
   
   if (t  < 1.0)
     e_add_event_timer("e_list_item_zoom()", 0.02, e_list_item_zoom, v + 1, data);
}


void
e_list_item_in(void *_data, Evas _e, Evas_Object _o, int _b, int _x, int _y)
{
   e_list_item_zoom(0, _data);
}

void
e_list_item_out(void *_data, Evas _e, Evas_Object _o, int _b, int _x, int _y)
{
   e_list_item_zoom(-1 , _data);
}

void
e_idle(void *data)
{
   if (need_thumbs)
     {
	if (!generating_image)
	  {
	     Evas_List l;
	     int doing;
	     int i;
	     
	     doing = 0;
	     i = 1;
	     for (l = images; l; l = l->next, i++)
	       {
		  Image *im;
		  
		  im = l->data;
		  if ((im->subst) && (im->thumb))
		    {
		       int w, h;
		       Imlib_Load_Error err;
		       
		       evas_del_object(evas, im->o_thumb);
		       if (e_file_is_dir(im->file))
			 im->o_thumb = evas_add_image_from_file(evas,
								IM"directory.png");
		       else
			 im->o_thumb = evas_add_image_from_file(evas, 
								im->thumb);
		       evas_callback_add(evas, im->o_thumb, CALLBACK_MOUSE_DOWN, e_list_click, l);
		       evas_callback_add(evas, im->o_thumb, CALLBACK_MOUSE_IN, e_list_item_in, l);
		       evas_callback_add(evas, im->o_thumb, CALLBACK_MOUSE_OUT, e_list_item_out, l);
		       evas_get_image_size(evas, im->o_thumb, &w, &h);
		       im->subst = 0;
		       evas_move(evas, im->o_thumb, 
				 2 + ((48 - w) / 2) + icon_x, 
				 2 + ((48 + 2) * (i - 1)) + ((48 - h) / 2) + icon_y);
		       evas_set_layer(evas, im->o_thumb, 210);
		       evas_show(evas, im->o_thumb);
		       im->subst = 0;
		    }
		  if (!im->thumb) 
		    {
		       e_generate_thumb(im);
		       doing = 1;
		    }
	       }
	     if (!doing) need_thumbs = 0;
	  }
	e_fix_icons();
     }
   evas_render(evas);
}

void 
e_window_expose(Eevent * ev)
{
   Ev_Window_Expose      *e;
   
   e = (Ev_Window_Expose *)ev->event;
   if ((e->win != evas_get_window(evas)))
     return;
   evas_update_rect(evas, e->x, e->y, e->w, e->h);
}

void 
e_mouse_move(Eevent * ev)
{
   Ev_Mouse_Move      *e;
   
   e = (Ev_Mouse_Move *)ev->event;
   if ((e->win != evas_get_window(evas)))
     return;
   evas_event_move(evas, e->x, e->y);
}

void 
e_mouse_down(Eevent * ev)
{
   Ev_Mouse_Down      *e;
   
   e = (Ev_Mouse_Down *)ev->event;
   if ((e->win != evas_get_window(evas)))
     return;
   evas_event_button_down(evas, e->x, e->y, e->button);
}

void 
e_mouse_up(Eevent * ev)
{
   Ev_Mouse_Up      *e;
   
   e = (Ev_Mouse_Up *)ev->event;
   if ((e->win != evas_get_window(evas)))
     return;
   evas_event_button_up(evas, e->x, e->y, e->button);
}

void 
e_window_configure(Eevent * ev)
{
   Ev_Window_Configure *e;
   
   e = (Ev_Window_Configure *)ev->event;
   if (e->win == main_win)
     {
	win_w = e->w;
	win_h = e->h;
	if (e->wm_generated)
	  {
	     win_x = e->x;
	     win_y = e->y;
	  }
	e_window_resize(evas_get_window(evas), win_w, win_h);
	evas_set_output_viewport(evas, 0, 0, win_w, win_h);
	evas_set_output_size(evas, win_w, win_h);
	e_handle_resize();
	if (panel_active)
	  e_slide_panel_out(0, NULL);
	if (buttons_active)
	  e_slide_buttons_out(0, NULL);
     }
}

void 
e_key_down(Eevent * ev)
{
   Ev_Key_Down          *e;
   
   e = ev->event;
   if ((e->win != main_win) &&
       (e->win != evas_get_window(evas)))
     return;
   if (!strcmp(e->key, "n"))
     {
	scale = 1.0;
	e_handle_resize();
     }
   else if (!strcmp(e->key, "minus"))
     {
	scale += 0.5;
	e_handle_resize();
     }
   else if (!strcmp(e->key, "equal"))
     {
	scale -= 0.5;
	if (scale < 0.5) scale = 0.5;
	e_handle_resize();
     }
   else if (!strcmp(e->key, "w"))
     {
	int w, h;
	
	if (o_image)
	  {
	     double sh, sv;
	     
	     evas_get_image_size(evas, o_image, &w, &h);
	     sh = (double)w / (double)win_w;
	     sv = (double)h / (double)win_h;
	     scale = sv;
	     if (sh > sv) scale = sh;
	     e_handle_resize();
	  }
     }
   else if (!strcmp(e->key, "f"))
     {
	e_toggle_fullscreen();
     }
   else if (!strcmp(e->key, "i"))
     {
	e_fade_info_in(0, NULL);
     }
   else if (!strcmp(e->key, "h"))
     {
	/* FIXME: display help */
     }
   else if (!strcmp(e->key, "q"))
     {
	exit(0);
     }
   else if (!strcmp(e->key, "Escape"))
     {
	exit(0);
     }
   else if (!strcmp(e->key, "Return"))
     {
     }
   else if (!strcmp(e->key, "space"))
     {
	if (!current_image)
	  current_image = images;
	else
	  {
	     if (current_image->next)
	       current_image = current_image->next;
	  }
	e_display_current_image();
     }
   else if (!strcmp(e->key, "BackSpace"))
     {
	if (!current_image)
	  current_image = images;
	else
	  current_image = current_image->prev;
	e_display_current_image();
     }
   else if (!strcmp(e->key, "Up"))
     {
	icon_y += 8;
	e_fix_icons();
     }
   else if (!strcmp(e->key, "Down"))
     {
	icon_y -= 8;
	e_fix_icons();
     }
}

void 
e_property(Eevent * ev)
{
   Ev_Window_Property      *e;
   Atom                     a_entice_newfiles = 0;
   char                    *files;
   int                      size;
   
   e = (Ev_Window_Property *)ev->event;
   if ((e->win != main_win))
     return;
   if (!a_entice_newfiles) a_entice_newfiles = e_atom_get("_ENTICE_NEWFILES");
   if (e->atom != a_entice_newfiles) return;
   files = e_window_property_get(e->win, e->atom, XA_STRING, &size);
   if (files)
     {
	char file[4096], *p, *pp;
	
	pp = files;
	while ((p = strchr(pp, '\n')))
	  {	
	     Image *im;
	     
	     *p = 0;
	     strcpy(file, pp);
	     im = e_image_new(file);
	     images = evas_list_append(images, im);
	     pp = p + 1;
	     if (pp >= files + size) break;
	  }
	free(files);
	  {
	     Evas_List l;
	     int i;
	     
	     i = 1;
	     for (l = images; l; l = l->next, i++)
	       {
		  Image *im;
		  int first;
		  
		  im = l->data;
		  first = 1;
		  if (!im->o_thumb)
		    {
		       if (first)
			 {
			    current_image = l;
			    first = 0;
			 }
		       im->o_thumb = evas_add_image_from_file(evas, IM"thumb.png");
		       evas_callback_add(evas, im->o_thumb, CALLBACK_MOUSE_DOWN, e_list_click, l);
		       evas_callback_add(evas, im->o_thumb, CALLBACK_MOUSE_IN, e_list_item_in, l);
		       evas_callback_add(evas, im->o_thumb, CALLBACK_MOUSE_OUT, e_list_item_out, l);
		       im->subst = 1;
		       evas_set_image_border(evas, im->o_thumb, 4, 4, 4, 4);
		       evas_move(evas, im->o_thumb, 2, 2 + ((48 + 2) * (i - 1)));
		       evas_resize(evas, im->o_thumb, 48, 48);
		       evas_set_image_fill(evas, im->o_thumb, 0, 0, 48, 48);
		       evas_set_layer(evas, im->o_thumb, 210);
		       evas_show(evas, im->o_thumb);
		    }
	       }
	  }
	need_thumbs = 1;
	e_fix_icons();
	e_display_current_image();
     }
}

/* callbacks evas will call for events within the evas */

void
show_panel (void *_data, Evas _e, Evas_Object _o, int _b, int _x, int _y)
{
   if (!panel_active)
     e_slide_panel_in(0, NULL);
}

void
hide_panel (void *_data, Evas _e, Evas_Object _o, int _b, int _x, int _y)
{
   if (panel_active)
     e_slide_panel_out(0, NULL);
   if (buttons_active)
     e_slide_buttons_out(0, NULL);
}

void
show_buttons (void *_data, Evas _e, Evas_Object _o, int _b, int _x, int _y)
{
   if (!buttons_active)
     e_slide_buttons_in(0, NULL);
}

void
next_image(void *_data, Evas _e, Evas_Object _o, int _b, int _x, int _y)
{
   down_x = _x;
   down_y = _y;
   down_sx = scroll_x;
   down_sy = scroll_y;
   e_fade_scroller_in(0, NULL);
}

void
next_image_up(void *_data, Evas _e, Evas_Object _o, int _b, int _x, int _y)
{
   if (((_x - down_x) * (_x - down_x)) +
       ((_y - down_y) * (_y - down_y)) > 9)
     {
	scroll_x = scroll_sx;
	scroll_y = scroll_sy;
	e_fade_scroller_out(0, NULL);
	return;
     }
   if ((_o == o_showpanel) && (panel_active)) return;
   if (!current_image)
     current_image = images;
   else
     {
	if ((_b == 1) && (current_image->next))
	  current_image = current_image->next;
	else if (_b == 3)
	  current_image = current_image->prev;
     }
   e_display_current_image();
}

void
next_image_move(void *_data, Evas _e, Evas_Object _o, int _b, int _x, int _y)
{
   if (_b != 0)
     {
	scroll_x = _x - down_x + down_sx;
	scroll_y = _y - down_y + down_sy;
	
	e_handle_resize();
     }
}

void
e_list_scroll_up_timer(int v, void *data)
{
   int vv;
   
   vv = v / 5;
   if (vv > 15) vv = 15;
   icon_y -= 1 + vv;
   e_fix_icons();
   e_add_event_timer("e_list_scroll_up_timer()", 0.02, e_list_scroll_up_timer, v + 1, NULL);
}

void
e_list_scroll_down_timer(int v, void *data)
{
   int vv;
   
   vv = v / 5;
   if (vv > 15) vv = 15;
   icon_y += 1 + vv;
   e_fix_icons();
   e_add_event_timer("e_list_scroll_down_timer()", 0.02, e_list_scroll_down_timer, v + 1, NULL);
}

void
bt_tip_start (void *_data, Evas _e, Evas_Object _o, int _b, int _x, int _y)
{
   int i;
      
   for (i = 0; i < 5; i++)
     {
	double tw, th, x, y;
	
	evas_show(evas, o_txt_tip[i]);
	evas_set_layer(evas, o_txt_tip[i], 2000);
	evas_set_text(evas, o_txt_tip[i], _data);
	tw = evas_get_text_width(evas, o_txt_tip[i]);
	th = evas_get_text_height(evas, o_txt_tip[i]);
	x = ((double)win_w - tw) / 2;
	y = ((double)win_h - th) / 2;
	if      (i == 0)
	  evas_move(evas, o_txt_tip[i], x - 1, y + 0);
	else if (i == 1)
	  evas_move(evas, o_txt_tip[i], x + 0, y - 1);
	else if (i == 2)
	  evas_move(evas, o_txt_tip[i], x + 1, y + 0);
	else if (i == 3)
	  evas_move(evas, o_txt_tip[i], x + 0, y + 1);
	else if (i == 4)
	  evas_move(evas, o_txt_tip[i], x + 0, y + 0);
	if (i == 4)
	  evas_set_color(evas, o_txt_tip[i], 255, 255, 255, 255);
	else
	  evas_set_color(evas, o_txt_tip[i], 0, 0, 0, 255);
     }
}

void
bt_tip_stop (void *_data, Evas _e, Evas_Object _o, int _b, int _x, int _y)
{
   int i;
      
   for (i = 0; i < 5; i++) evas_hide(evas, o_txt_tip[i]);
}

void
bt_close_down (void *_data, Evas _e, Evas_Object _o, int _b, int _x, int _y)
{
   evas_set_image_file(_e, _o, IM"bt_close_2.png");
}

void
bt_close_up(void *_data, Evas _e, Evas_Object _o, int _b, int _x, int _y)
{
   evas_set_image_file(_e, _o, IM"bt_close_1.png");
   exit(0);
}

void
bt_expand_down (void *_data, Evas _e, Evas_Object _o, int _b, int _x, int _y)
{
   evas_set_image_file(_e, _o, IM"bt_expand_2.png");
}

void
bt_expand_up(void *_data, Evas _e, Evas_Object _o, int _b, int _x, int _y)
{
   evas_set_image_file(_e, _o, IM"bt_expand_1.png");
   if (o_image)
     {
	double sh, sv;
	int w, h;
	
	evas_get_image_size(evas, o_image, &w, &h);
	sh = (double)w / (double)win_w;
	sv = (double)h / (double)win_h;
	scale = sv;
	if (sh > sv) scale = sh;
	e_handle_resize();
     }
}

void
bt_full_down (void *_data, Evas _e, Evas_Object _o, int _b, int _x, int _y)
{
   evas_set_image_file(_e, _o, IM"bt_full_2.png");
}

void
bt_full_up(void *_data, Evas _e, Evas_Object _o, int _b, int _x, int _y)
{
   evas_set_image_file(_e, _o, IM"bt_full_1.png");
   e_toggle_fullscreen();
}

void
bt_next_down (void *_data, Evas _e, Evas_Object _o, int _b, int _x, int _y)
{
   evas_set_image_file(_e, _o, IM"bt_next_2.png");
}

void
bt_next_up(void *_data, Evas _e, Evas_Object _o, int _b, int _x, int _y)
{
   evas_set_image_file(_e, _o, IM"bt_next_1.png");
   if (!current_image)
     current_image = images;
   else
     {
	if (current_image->next)
	  current_image = current_image->next;
     }
   e_display_current_image();
}

void
bt_prev_down (void *_data, Evas _e, Evas_Object _o, int _b, int _x, int _y)
{
   evas_set_image_file(_e, _o, IM"bt_prev_2.png");
}

void
bt_prev_up(void *_data, Evas _e, Evas_Object _o, int _b, int _x, int _y)
{
   evas_set_image_file(_e, _o, IM"bt_prev_1.png");
   if (!current_image)
     current_image = images;
   else
     current_image = current_image->prev;
   e_display_current_image();
}

void
bt_zoom_in_down (void *_data, Evas _e, Evas_Object _o, int _b, int _x, int _y)
{
   evas_set_image_file(_e, _o, IM"bt_zoom_in_2.png");
}

void
bt_zoom_in_up(void *_data, Evas _e, Evas_Object _o, int _b, int _x, int _y)
{
   evas_set_image_file(_e, _o, IM"bt_zoom_in_1.png");
   scale -= 0.5;
   if (scale < 0.5) scale = 0.5;
   e_handle_resize();
}

void
bt_zoom_normal_down (void *_data, Evas _e, Evas_Object _o, int _b, int _x, int _y)
{
   evas_set_image_file(_e, _o, IM"bt_zoom_normal_2.png");
}

void
bt_zoom_normal_up(void *_data, Evas _e, Evas_Object _o, int _b, int _x, int _y)
{
   evas_set_image_file(_e, _o, IM"bt_zoom_normal_1.png");
   scale = 1.0;
   e_handle_resize();
}

void
bt_zoom_out_down (void *_data, Evas _e, Evas_Object _o, int _b, int _x, int _y)
{
   evas_set_image_file(_e, _o, IM"bt_zoom_out_2.png");
}

void
bt_zoom_out_up(void *_data, Evas _e, Evas_Object _o, int _b, int _x, int _y)
{
   evas_set_image_file(_e, _o, IM"bt_zoom_out_1.png");
   scale += 0.5;
   e_handle_resize();
}

void
list_scroll_up (void *_data, Evas _e, Evas_Object _o, int _b, int _x, int _y)
{
   e_list_scroll_down_timer(0, NULL);
   evas_set_image_file(_e, _o, IM"list_arrow_u_2.png");
}

void
list_scroll_up_up (void *_data, Evas _e, Evas_Object _o, int _b, int _x, int _y)
{
   e_del_event_timer("e_list_scroll_down_timer()");
   evas_set_image_file(_e, _o, IM"list_arrow_u_1.png");
}

void
list_scroll_down (void *_data, Evas _e, Evas_Object _o, int _b, int _x, int _y)
{
   e_list_scroll_up_timer(0, NULL);
   evas_set_image_file(_e, _o, IM"list_arrow_d_2.png");
}

void
list_scroll_down_up (void *_data, Evas _e, Evas_Object _o, int _b, int _x, int _y)
{
   e_del_event_timer("e_list_scroll_up_timer()");
   evas_set_image_file(_e, _o, IM"list_arrow_d_1.png");
}

void
mouse_down(void *_data, Evas _e, Evas_Object _o, int _b, int _x, int _y)
{
}

void
mouse_up (void *_data, Evas _e, Evas_Object _o, int _b, int _x, int _y)
{
}

void
mouse_move (void *_data, Evas _e, Evas_Object _o, int _b, int _x, int _y)
{
}

void
mouse_in (void *_data, Evas _e, Evas_Object _o, int _b, int _x, int _y)
{
}

void
mouse_out (void *_data, Evas _e, Evas_Object _o, int _b, int _x, int _y)
{
}

/* utils */
double
get_time(void)
{
   struct timeval      timev;
   
   gettimeofday(&timev, NULL);
   return (double)timev.tv_sec + (((double)timev.tv_usec) / 1000000);
}

/* meat */
void
setup(void)
{
   Window win, ewin;
   int i, j;
   Atom a_entice;
   char string[] = "entice";
   
   /* setup callbacks for events */
   e_event_filter_handler_add(EV_WINDOW_EXPOSE,            e_window_expose);
   e_event_filter_handler_add(EV_MOUSE_MOVE,               e_mouse_move);
   e_event_filter_handler_add(EV_MOUSE_DOWN,               e_mouse_down);
   e_event_filter_handler_add(EV_MOUSE_UP,                 e_mouse_up);
   e_event_filter_handler_add(EV_WINDOW_CONFIGURE,         e_window_configure);
   e_event_filter_handler_add(EV_KEY_DOWN,                 e_key_down);
   e_event_filter_handler_add(EV_WINDOW_PROPERTY,          e_property);
   /* handler for when the event queue goes idle */
   e_event_filter_idle_handler_add(e_idle, NULL);
   /* create a 400x300 toplevel window */
   win = e_window_new(0, 0, 0, win_w, win_h);
   e_window_set_events(win, XEV_CONFIGURE | XEV_PROPERTY);
   e_window_set_name_class(win, "Entice", "Main");
   e_window_set_min_size(win, 256, 128);
   e_window_set_max_size(win, 8000, 8000);
   a_entice = e_atom_get("_ENTICE_APP_WINDOW");
   e_window_property_set(win, a_entice, XA_STRING, 8, string, 6);
   main_win = win;
   
   /* create a 400x300 evas rendering in software - conveience function that */
   /* also creates the window for us in the right colormap & visual */
   evas = evas_new_all(e_display_get(), win, 0, 0, win_w, win_h, render_method,
		       max_colors, MAX_FONT_CACHE, MAX_IMAGE_CACHE,
		       FONT_DIRECTORY);
   /* tell evas if we want our image scaled with "smooth" scaling or not */
   evas_set_scale_smoothness(evas, smoothness);
   /* get the window ID for the evas created for us */
   ewin = evas_get_window(evas);
  
   /* show the evas window */
   e_window_show(ewin);
   /* set the events this window accepts */
   e_window_set_events(ewin, XEV_EXPOSE | XEV_BUTTON | XEV_MOUSE_MOVE | XEV_KEY);
   /* show the toplevel */
   e_window_show(win);
   
   /* now... create objects in the evas */
   
   o_bg = evas_add_image_from_file(evas, IM"bg.png");   
   o_logo = evas_add_image_from_file(evas, IM"logo.png");
   o_panel = evas_add_image_from_file(evas, IM"panel.png");
   o_showpanel = evas_add_rectangle(evas);
   o_hidepanel = evas_add_rectangle(evas);
   o_showbuttons = evas_add_rectangle(evas);
   o_arrow_l = evas_add_image_from_file(evas, IM"arrow_l.png");
   o_arrow_r = evas_add_image_from_file(evas, IM"arrow_r.png");
   o_arrow_t = evas_add_image_from_file(evas, IM"arrow_t.png");
   o_arrow_b = evas_add_image_from_file(evas, IM"arrow_b.png");
   o_mini_select = evas_add_image_from_file(evas, IM"mini_sel.png");
   o_list_select = evas_add_image_from_file(evas, IM"list_sel.png");
   o_panel_arrow_u = evas_add_image_from_file(evas, IM"list_arrow_u_1.png");
   o_panel_arrow_d = evas_add_image_from_file(evas, IM"list_arrow_d_1.png");
   o_bt_close = evas_add_image_from_file(evas, IM"bt_close_1.png");
   o_bt_expand = evas_add_image_from_file(evas, IM"bt_expand_1.png");
   o_bt_full = evas_add_image_from_file(evas, IM"bt_full_1.png");
   o_bt_next = evas_add_image_from_file(evas, IM"bt_next_1.png");
   o_bt_prev = evas_add_image_from_file(evas, IM"bt_prev_1.png");
   o_bt_zoom_in = evas_add_image_from_file(evas, IM"bt_zoom_in_1.png");
   o_bt_zoom_normal = evas_add_image_from_file(evas, IM"bt_zoom_normal_1.png");
   o_bt_zoom_out = evas_add_image_from_file(evas, IM"bt_zoom_out_1.png");
   evas_set_color(evas, o_logo, 255, 255, 255, 0);
   evas_set_color(evas, o_showpanel, 0, 0, 0, 0);
   evas_set_color(evas, o_hidepanel, 0, 0, 0, 0);
   evas_set_color(evas, o_showbuttons, 0, 0, 0, 0);
   evas_set_pass_events(evas, o_panel, 1);
   for (j = 0; j < 2; j++)
     {
	for (i = 0; i < 5; i++)
	  {
	     o_txt_info[(j * 5) + i] = evas_add_text(evas, "nationff", 16, "");
	     evas_set_pass_events(evas, o_txt_info[(j * 5) + i], 1);
	  }	
     }
   for (i = 0; i < 5; i++)
     {
	o_txt_tip[i] = evas_add_text(evas, "nationff", 16, "");
	evas_set_pass_events(evas, o_txt_info[i], 1);
     }
   
   e_display_current_image();
   
   evas_show(evas, o_bg);
   evas_show(evas, o_logo);
   evas_show(evas, o_panel);
   evas_show(evas, o_showpanel);
   evas_show(evas, o_hidepanel);
   evas_show(evas, o_showbuttons);

   evas_show(evas, o_panel_arrow_u);
   evas_show(evas, o_panel_arrow_d);

   evas_show(evas, o_bt_close);
   evas_show(evas, o_bt_expand);
   evas_show(evas, o_bt_full);
   evas_show(evas, o_bt_next);
   evas_show(evas, o_bt_prev);
   evas_show(evas, o_bt_zoom_in);
   evas_show(evas, o_bt_zoom_normal);
   evas_show(evas, o_bt_zoom_out);

   e_slide_panel_out(0, NULL);
   e_slide_buttons_out(0, NULL);

   evas_callback_add(evas, o_showpanel, CALLBACK_MOUSE_IN, show_panel, NULL);
   evas_callback_add(evas, o_hidepanel, CALLBACK_MOUSE_IN, hide_panel, NULL);
   evas_callback_add(evas, o_showbuttons, CALLBACK_MOUSE_IN, show_buttons, NULL);
   evas_callback_add(evas, o_logo, CALLBACK_MOUSE_DOWN, next_image, NULL);
   evas_callback_add(evas, o_logo, CALLBACK_MOUSE_UP, next_image_up, NULL);
   evas_callback_add(evas, o_bg, CALLBACK_MOUSE_DOWN, next_image, NULL);
   evas_callback_add(evas, o_bg, CALLBACK_MOUSE_UP, next_image_up, NULL);
   evas_callback_add(evas, o_hidepanel, CALLBACK_MOUSE_DOWN, next_image, NULL);
   evas_callback_add(evas, o_hidepanel, CALLBACK_MOUSE_UP, next_image_up, NULL);
   evas_callback_add(evas, o_hidepanel, CALLBACK_MOUSE_MOVE, next_image_move, NULL);
   evas_callback_add(evas, o_showpanel, CALLBACK_MOUSE_DOWN, next_image, NULL);
   evas_callback_add(evas, o_showpanel, CALLBACK_MOUSE_UP, next_image_up, NULL);
   evas_callback_add(evas, o_showpanel, CALLBACK_MOUSE_MOVE, next_image_move, NULL);
   evas_callback_add(evas, o_panel_arrow_u, CALLBACK_MOUSE_DOWN, list_scroll_up, NULL);
   evas_callback_add(evas, o_panel_arrow_u, CALLBACK_MOUSE_UP, list_scroll_up_up, NULL);
   evas_callback_add(evas, o_panel_arrow_d, CALLBACK_MOUSE_DOWN, list_scroll_down, NULL);
   evas_callback_add(evas, o_panel_arrow_d, CALLBACK_MOUSE_UP, list_scroll_down_up, NULL);
   evas_callback_add(evas, o_bt_close, CALLBACK_MOUSE_DOWN, bt_close_down, NULL);
   evas_callback_add(evas, o_bt_close, CALLBACK_MOUSE_UP, bt_close_up, NULL);
   evas_callback_add(evas, o_bt_close, CALLBACK_MOUSE_IN, bt_tip_start, "Exit [ Key: Q ]");
   evas_callback_add(evas, o_bt_close, CALLBACK_MOUSE_OUT, bt_tip_stop, NULL);
   evas_callback_add(evas, o_bt_expand, CALLBACK_MOUSE_DOWN, bt_expand_down, NULL);
   evas_callback_add(evas, o_bt_expand, CALLBACK_MOUSE_UP, bt_expand_up, NULL);
   evas_callback_add(evas, o_bt_expand, CALLBACK_MOUSE_IN, bt_tip_start, "Scale image to fit entirely in view [ Key: W ]");
   evas_callback_add(evas, o_bt_expand, CALLBACK_MOUSE_OUT, bt_tip_stop, NULL);
   evas_callback_add(evas, o_bt_full, CALLBACK_MOUSE_DOWN, bt_full_down, NULL);
   evas_callback_add(evas, o_bt_full, CALLBACK_MOUSE_UP, bt_full_up, NULL);
   evas_callback_add(evas, o_bt_full, CALLBACK_MOUSE_IN, bt_tip_start, "Toggle fullscreen display [ Key: F ]");
   evas_callback_add(evas, o_bt_full, CALLBACK_MOUSE_OUT, bt_tip_stop, NULL);
   evas_callback_add(evas, o_bt_next, CALLBACK_MOUSE_DOWN, bt_next_down, NULL);
   evas_callback_add(evas, o_bt_next, CALLBACK_MOUSE_UP, bt_next_up, NULL);
   evas_callback_add(evas, o_bt_next, CALLBACK_MOUSE_IN, bt_tip_start, "View next image [ Key: Space ]");
   evas_callback_add(evas, o_bt_next, CALLBACK_MOUSE_OUT, bt_tip_stop, NULL);
   evas_callback_add(evas, o_bt_prev, CALLBACK_MOUSE_DOWN, bt_prev_down, NULL);
   evas_callback_add(evas, o_bt_prev, CALLBACK_MOUSE_UP, bt_prev_up, NULL);
   evas_callback_add(evas, o_bt_prev, CALLBACK_MOUSE_IN, bt_tip_start, "View previous image [ Key: Backspace ]");
   evas_callback_add(evas, o_bt_prev, CALLBACK_MOUSE_OUT, bt_tip_stop, NULL);
   evas_callback_add(evas, o_bt_zoom_in, CALLBACK_MOUSE_DOWN, bt_zoom_in_down, NULL);
   evas_callback_add(evas, o_bt_zoom_in, CALLBACK_MOUSE_UP, bt_zoom_in_up, NULL);
   evas_callback_add(evas, o_bt_zoom_in, CALLBACK_MOUSE_IN, bt_tip_start, "Zoom in one step [ Key: = ]");
   evas_callback_add(evas, o_bt_zoom_in, CALLBACK_MOUSE_OUT, bt_tip_stop, NULL);
   evas_callback_add(evas, o_bt_zoom_normal, CALLBACK_MOUSE_DOWN, bt_zoom_normal_down, NULL);
   evas_callback_add(evas, o_bt_zoom_normal, CALLBACK_MOUSE_UP, bt_zoom_normal_up, NULL);
   evas_callback_add(evas, o_bt_zoom_normal, CALLBACK_MOUSE_IN, bt_tip_start, "Reset zoom to normal 1:1  [ Key: N ]");
   evas_callback_add(evas, o_bt_zoom_normal, CALLBACK_MOUSE_OUT, bt_tip_stop, NULL);
   evas_callback_add(evas, o_bt_zoom_out, CALLBACK_MOUSE_DOWN, bt_zoom_out_down, NULL);
   evas_callback_add(evas, o_bt_zoom_out, CALLBACK_MOUSE_UP, bt_zoom_out_up, NULL);
   evas_callback_add(evas, o_bt_zoom_out, CALLBACK_MOUSE_IN, bt_tip_start, "Zoom out one step [ Key: - ]");
   evas_callback_add(evas, o_bt_zoom_out, CALLBACK_MOUSE_OUT, bt_tip_stop, NULL);
}

Window
find_current_rep(Window win, Atom atom)
{
   Window *wlist;
   int i, n;
   
   wlist = e_window_get_children(win, &n);
   if (wlist)
     {
	for (i = 0; i < n; i++)
	  {
	     void *data;
	     int size;
	     Window w;
	     
	     w = wlist[i];
	     data = e_window_property_get(w, atom, XA_STRING, &size);
	     if (data)
	       {
		  free(data);
		  free(wlist);
		  return w;
	       }
	     else
	       {
		  Window ww;
		  
		  ww = find_current_rep(w, atom);
		  if (ww)
		    {
		       free(wlist);
		       return ww;
		    }
	       }
	  }
	free(wlist);
     }
   return 0;
}

void
find_current(void)
{
   Atom a_entice;
   Atom a_entice_newfiles;
   Window win;
   Evas_List l;
   int size;
   char *files;
   
   if (!images) return;
   a_entice = e_atom_get("_ENTICE_APP_WINDOW");
   win = find_current_rep(0, a_entice);
   if (!win) return;
   a_entice_newfiles = e_atom_get("_ENTICE_NEWFILES");
   size = 0;
   for (l = images; l; l = l->next)
     {
	Image *im;
	
	im = l->data;
	size += strlen(im->file) + 1;
     }
   files = malloc(size);
   files[0] = 0;
   for (l = images; l; l = l->next)
     {
	Image *im;
	
	im = l->data;
	strcat(files, im->file);
	strcat(files, "\n");
     }
   e_window_property_set(win, a_entice_newfiles, XA_STRING, 8, files, size);
   e_sync();
   e_sync();
   exit(0);
}

int 
main(int argc, char **argv)
{
   /* environment parsing */
   if (getenv("ENTICE_RENDERER"))
     {
	char *entice_renderer;
	
	entice_renderer = getenv("ENTICE_RENDERER");
	if (!strcmp(entice_renderer, "soft")) 
	  render_method = RENDER_METHOD_ALPHA_SOFTWARE;
	else if (!strcmp(entice_renderer, "x11")) 
	  render_method = RENDER_METHOD_BASIC_HARDWARE;
	else if (!strcmp(entice_renderer, "hard")) 
	  render_method = RENDER_METHOD_3D_HARDWARE;	
     }
   if (getenv("ENTICE_COLORS"))
     {
	char *entice_colors;
	
	entice_colors = getenv("EVS_COLORS");
	max_colors = atoi(entice_colors);
     }
   if (getenv("ENTICE_SMOOTH"))
     {
	char *entice_smooth;
	
	entice_smooth = getenv("EVS_SMOOTH");
	smoothness = atoi(entice_smooth);
     }
   /* image list parsing */
     {
	int i;
	
	for (i = 1; i < argc; i++)
	  {
	     Image *im;
	     
	     if (argv[i][0] == '/')
	       im = e_image_new(argv[i]);
	     else
	       {
		  char buf[4096];
		  char wd[4096];
		  
		  getcwd(wd, sizeof(wd));		  
		  sprintf(buf, "%s/%s", wd, argv[i]);
		  im = e_image_new(buf);		  
	       }
	     images = evas_list_append(images, im);
	  }
	current_image = images;
     }
   /* init X */
   if (!e_display_init(NULL))
     {
	if (getenv("DISPLAY"))
	  {
	     printf("Cannot initialize default display:\n");
	     printf("DISPLAY=%s\n", getenv("DISPLAY"));
	  }
	else
	  {
	     printf("No DISPLAY variable set!\n");
	  }
	printf("Exit.\n");
	exit(-1);
     }
   /* setup handlers for system signals */
   e_ev_signal_init();
   /* setup the event filter */
   e_event_filter_init();
   /* setup the X event internals */
   e_ev_x_init();
   
   /* find if another entice is running... if it is .. message it. */
   find_current();

   /* program does its data setup here */
   setup();
   
   /* setup thumbnails */
     {
	Evas_List l;
	int i;
	
	i = 1;
	for (l = images; l; l = l->next, i++)
	  {
	     Image *im;
	     
	     im = l->data;
	     im->o_thumb = evas_add_image_from_file(evas, IM"thumb.png");
	     evas_callback_add(evas, im->o_thumb, CALLBACK_MOUSE_DOWN, e_list_click, l);
	     evas_callback_add(evas, im->o_thumb, CALLBACK_MOUSE_IN, e_list_item_in, l);
	     evas_callback_add(evas, im->o_thumb, CALLBACK_MOUSE_OUT, e_list_item_out, l);
	     im->subst = 1;
	     evas_set_image_border(evas, im->o_thumb, 4, 4, 4, 4);
	     evas_move(evas, im->o_thumb, 2, 2 + ((48 + 2) * (i - 1)));
	     evas_resize(evas, im->o_thumb, 48, 48);
	     evas_set_image_fill(evas, im->o_thumb, 0, 0, 48, 48);
	     evas_set_layer(evas, im->o_thumb, 210);
	     evas_show(evas, im->o_thumb);
	  }
     }
   icon_x = -100;
   e_fix_icons();
/*   
     {
	Evas_List l;
	
	for (l = images; l; l = l->next)
	  {
	     current_image = l;
	     e_display_current_image();
	     evas_render(evas);
	  }
     }
   exit(0);
*/   
   /* call the animator once to start it up */
   e_fade_logo_in(0, NULL);
   e_fade_scroller_in(0, (void *)1);   
   /* and now loop forever handling events */
   e_event_loop();
}
